function [ridge_points, pred] = rsridge(y, D, d0, minmax, step_len, prefix, num_steps)
% RSRIDGE - Fit a full quadratic model and suggest measurements using ridge analysis. Version 0.1.22.
%
% The syntax is
% [ridge_points, pred] = rsridge(y, D, d0, minmax, step_len, [prefix], [num_steps])
%
% Arguments
% y is an N by 1 column vector of measured response data.
% D is an N by k design matrix with factor data.
% d0 is the midpoint of the (hyper)spheres on whose surfaces optima are picked out.
% minmax should be set to 0 if the aim is to minimize the response, otherwise 1.
% step_len is the difference of the radii of the spheres where the optimum is chosen.
% [prefix] is a voluntary string. If k=2 and [prefix] is passed to rsridge then a pdf file is saved with the suggested points plotted in a contour plot.
% [num_steps] is a voluntary parameter indicating the number of steps to output. Default is 10.
%
% Return values
% ridge_points is a list of ridge optima where one should make measurements.
% pred is list of the corresponding predictions by the model (not likely to match actual measurements further away).
% 
%
% References
% Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009, Section 6.3.3 and 6.4.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 5 || nargin > 7
    error('Try help rsridge for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsridge for instructions.');
end
if nargin<7
  num_steps=10;
end

% Fit quadratic model
k=size(D,2);
X=rsquad(D);
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix D does not have linearly dependent columns or a column of ones, try standardizing the factor data.');
end
b=X\y;
b0=b(1);
b1=b(2:(k+1));
b(1:(k+1))=[];
F=zeros(k,k);
for j=0:(k-1)
  F=F+diag(b(1:(k-j)),j);
  b(1:(k-j))=[];
end
B=(F+F')/2;

switch minmax
case 0
  mubounds=[-1e5 min(eig(B))-10*eps];
otherwise
  mubounds=[max(eig(B))+10*eps 1e5];
end

% Do ridge analysis
[Q,Lbda]=eig(B);
d0=d0';
b=X\y;
for m=1:num_steps
  rad=@(mu) norm((Lbda-mu*eye(k))^-1*Q'*(-b1/2-B*d0))-m*step_len;
  mu=fzero(rad,mubounds);
  di(:,m)=d0+(B-mu*eye(k))^-1*(-b1/2-B*d0);
  dit=di(:,m)';
  pred(m,1)=rsquad(dit)*b;
end
ridge_points=di';

% Draw contour plot, ridge points, and stationary point if k=2
if k==2
  z=-B^-1*b1/2;
  z=z';
  di=[d0 di z'];
  rscontour(y,D,[min(di(1,:))-1 max(di(1,:))+1 min(di(2,:))-1 max(di(2,:))+1])
  di=di(:,2:end-1);
  hold on
  plot(di(1,:),di(2,:),'kx','linewidth',2)
  plot(d0(1),d0(2),'b*','linewidth',2)
  plot(z(1),z(2),'rd','linewidth',2)
  hold off
  title('Ridge analysis with centre (blue) and stationary point (red)')
  
  if nargin > 5 && size(prefix,1)>0,
    print([prefix,'-rsridge'],'-dpdf')
  end
end

end